	
	list	p=16f877A	; list directive to define processor
	include	<p16F877A.inc>
    
	errorlevel -302,-207	; suppress message 302 from list file

	__config _CP_OFF & _WDT_OFF & _BODEN_OFF & _PWRTE_ON & _XT_OSC & _WRT_OFF & _LVP_OFF & _CPD_OFF

;===========================================================================  
;
;     PIC16F877-A   
;                                 __________  
;                 +5v------!MCLR |1       40| RB7-I---- (KEYPAD  COL4)  
;            (Unused)------  RA0 |2       39| RB6-I---- (KEYPAD  COL3)
;            (Unused)------  RA1 |3       38| RB5-I---- (KEYPAD  COL2)
;            (Unused)------  RA2 |4       37| RB4-I---- (KEYPAD  COL1)
;            (Unused)------  RA3 |5       36| RB3  ---- (Unused)  
;            (Unused)------  RA4 |6       35| RB2  ---- (Unused)  
;            (Unused)------  RA5 |7       34| RB1  ---- (Unused)                  
;            (Unused)------  RE0 |8       33| RB0/INT-- (Unused)  
;            (Unused)------  RE1 |9       32| VDD  ---- +5v  
;            (Unused)------  RE2 |10      31| VSS  ---- GND  
;                 +5v------  VDD |11      30| RD7-O---- (KEYPAD  ROW4)   
;                 GND------  VSS |12      29| RD6-O---- (KEYPAD  ROW3)
;                XTAL------ OSC1 |13      28| RD5-O---- (KEYPAD  ROW2)  
;                XTAL------ OSC2 |14      27| RD4-O---- (KEYPAD  ROW1) 
;             (Unused)-----  RC0 |15      26| RC7-O---- (LCD pin 14)  LCD_DB7    
; LCD_rs (LCD pin 4) ------O-RC1 |16      25| RC6-O---- (LCD pin 13)  LCD_DB6  
; LCD_rw (LCD pin 5) ------O-RC2 |17      24| RC5-O---- (LCD pin 12)  LCD_DB5  
; LCD_e  (LCD pin 6) ------O-RC3 |18      23| RC4-O---- (LCD pin 11)  LCD_DB4  
;            (Unused)------  RD0 |19      22| RD3  ---- (Unused)  
;            (Unused)------  RD1 |20      21| RD2  ---- (Unused)  
;                                 ----------  
;
;	LCD (Hitachi)  
;                                 ________  
;    		                 VSS |1       |   
;                            VDD |2       |   
;                 CONTRAST   VEE |3       |   
;                 RC1------  RS  |4       |   
;                 RC2------  R/W |5       |   
;                 RC3------  E   |6       |   
;                       (Unused) |7       |                   
;                       (Unused) |8       |   
;                       (Unused) |9       |   
;                       (Unused) |10      |  
;                 RC4------  DB4 |11      |    
;                 RC5------  DB5 |12      |   
;                 RC6------  DB6 |13      |      
;                 RC7------  DB7 |14      |     
;          BACKLIGHT POWER   VDD |15      |     
;                   GROUND   VSS |16      |   
;                                 --------  
;
;
;    KEYPAD
;                     RB4        RB5         RB6          RB7
;                      |          |           |            |
;                      |          |           |            |
;                      |          |           |            |
;
;                    COL 1       COL 2       COL 3        COL 4   
;                 +---------+  +--------+  +--------+   +--------+  
; RD4-------ROW 1 |    1    |  |   2    |  |   3    |   |   A    |
;                 +---------+  +--------+  +--------+   +--------+ 
;                 +---------+  +--------+  +--------+   +--------+  
; RD4-------ROW 2 |    4    |  |   5    |  |   6    |   |   B    |
;                 +---------+  +--------+  +--------+   +--------+ 
;                 +---------+  +--------+  +--------+   +--------+  
; RD4-------ROW 3 |    7    |  |   8    |  |   9    |   |   C    |
;                 +---------+  +--------+  +--------+   +--------+  
;                 +---------+  +--------+  +--------+   +--------+  
; RD4-------ROW 4 |    *    |  |   0    |  |   #    |   |   D    |
;                 +---------+  +--------+  +--------+   +--------+  
;
;
;===============================================================================  
;DECLARATIONS. 
;===============================================================================  
RESET_V	equ	0X0000		;ADDRESS OF THE RESET VECTOR.
ISRV_V	equ	0X0004		;ADDRESS OF INTERRUPT VECTOR.
OSC_FREQ equ	D'4000000'	;OSCILLATOR FREQUENCY IS 4 MHZ.

LCD_RS	equ	1
LCD_RW	equ	2
LCD_E	equ	3
LCDBUSY	equ	7

; PORTB BITS
RB7		EQU	7			; KEYBOARD, COL 4.
RB6		EQU	6			; KEYBOARD, COL 3.
RB5		EQU	5 			; KEYBOARD, COL 2.
RB4		EQU	4			; KEYBOARD, COL 1.

; PORTD BITS
RD7		EQU	7			; KEYBOARD, ROW 1.
RD6		EQU	6			; KEYBOARD, ROW 2.
RD5		EQU	5			; KEYBOARD, ROW 3.
RD4		EQU	4			; KEYBOARD, ROW 4.

RD3		EQU	3			; KEY CODE, BIT 3, USE 1K + LED TO VSS.
RD2		EQU	2			; KEY CODE, BIT 2, USE 1K + LED TO VSS.
RD1		EQU	1			; KEY CODE, BIT 1, USE 1K + LED TO VSS.
RD0		EQU	0			; KEY CODE, BIT 0, USE 1K + LED TO VSS.

	cblock	H'20'		;START DATA BLOCK.

;LCD VARIABLES.
LCD_CHAR				;THE BYTE REPRESENTING THE LCD CHARACTER. 
LCD_READ 				;VARIABLE CONTAINING THE READ FOR LCD_BUSY. 

LCD_VALUE 				;THE LCD RS LINE - FLAG VALUE. 
RS_VALUE 

;KEYPAD VARIABLES.
TEMP					; TEMP STORAGE.
LASTKEY					; LAST KEY PRESSED.
KEYCNT					; COPY OF LASTKEY.

;MISC VARIABLES.
_W         				; TEMPORARY STORE FOR W.
_STATUS       	 		; TEMPORARY STORE FOR STATUS.

;DELAY EQUATES.  
TIMER					;USED IN THE LCD BUSY ROUNTINE. 

DELAY_VAL 				;USED IN THE DELAY ROUTINES. 
X_DLY_VAL 				;USED IN THE DELAY ROUTINES. 

COUNT1
COUNTA
COUNTB

	endc				;END OF DATA BLOCK.

;===============================================================================  
; PROGRAM JUMP TO START.  
;===============================================================================  
	org	RESET_V			;RESET VECTOR LOCATION.

	
;===============================================================================  
; ADD THE "INCLUDE" FILES.  
;=============================================================================== 	
    include "BANK_ROUTINES.INC"
    include	"LCD_ROUTINES.INC"
    include "KEYPAD_ROUTINES.INC"
    	goto	START
    ;=============================================================================
;THIS IS THE INTERUPT ROUTINE.
;=============================================================================
INTERRUPT	
	org	ISRV_V			; Interrupt vector location

	BANK0
          	
	call 	DELAY20
			
	movlw   B'11110000'	;CHECK FOR KEY DOWN.
	movwf   TEMP  
	movf    PORTB,W
	subwf   TEMP,F
	btfsc	STATUS, Z
	goto    DONE		;ALL KEYS UP SO RETURN.
	call    GETKEY		;KEY PRESSED NOW IN W (0 - 15).
	movwf   LASTKEY		;SAVE IT.
	call	DISPLAY_LED		;DISPLAY KEYCODE ON RD0,1,2 AND 3.
			
DONE
	clrf    PORTB		;CLEAR PORT B.
	clrf    PORTD		;CLEAR PORT D.
	movf    PORTB,F
	swapf   STATUS,W	;RESTORE SAVED REGISTERS.
	movwf   STATUS
	swapf   _W,F
	swapf   _W,W
	        
	bcf     INTCON,RBIF;CLEAR Port B interrupt FLAG
	
	retfie
	
;===============================================================================  
;INITIALISE PIC MICRO CONTROLLER.  
;===============================================================================  
PIC_INIT
	clrf	STATUS		;DO INITIALIZATION, SELECT BANK 0
		
	clrf	INTCON		;CLEAR INT-FLAGS, DISABLE INTERRUPTS
	clrf	PCLATH		;KEEP IN LOWER 2KBYTE
						
						; NOTE: - Make all the ports low first.
	clrf	PORTA		
	clrf	PORTB	
	clrf	PORTC	
	clrf	PORTD	
	clrf	PORTE	

	BANK1				;CHANGE TO BANK1.

						; NOTE: - We now set each of the ports to either
						; INPUT or OUTPUT depending on eaches requirement.
								
	movlw   B'11110000'	
	movwf	TRISB		;NOW SET RB4-RB7 AS INPUTS  -> (KEYPAD). 	
	
	movlw	H'00'
	movwf	TRISC		;SET PINS RC0-7 TO BE OUTPUT -> (LCD).
				
	movlw	B'00000000'	; RD4-7 INPUTS  -> (KEYPAD).
	movwf	TRISD
			
	bcf		OPTION_REG, NOT_RBPU ;ENABLE WEAK PULL-UPS ON PORT B -> (KEYPAD)	

	BANK0				;CHANGE TO BANK0.

						; NOTE: - We now enable the INTERRUPTS.
													
	bcf		INTCON, INTF;CLEAR INTERRUPT FLAG.
	bsf		INTCON, RBIE;ENABLE INTERRUPT ON RB4-7 CHANGE.
	bsf		INTCON, GIE	;ENABLE INTERRUPTS.


	return





;===============================================================================  
; PROGRAM STARTS HERE.  
;===============================================================================  
START
	call	PIC_INIT	;INITIALIZE FOR RC0-7 TO BE OUTPUTS.
	call	LCD_INIT	;NOW INITIALIZE THE LCD FOR 4 BIT OPERATION.

	call	DISPLAY		;DISPLAY "HELLO".

LOOP	
	goto	LOOP
;===============================================================================  
; DISPLAY A TEST WORD ON THE LCD "HELLO"  
;===============================================================================  
DISPLAY

	movlw	'H'			;ENTER "H".
	call	LCD_DISP_CHAR

	movlw	'E'			;ENTER "E".
	call	LCD_DISP_CHAR

	movlw	'L'			;ENTER "L".
	call	LCD_DISP_CHAR

	movlw	'L'			;ENTER "L".
	call	LCD_DISP_CHAR

	movlw	'O'			;ENTER "O".
	call	LCD_DISP_CHAR
	


	movlw	H'6'
	call	LCD_LINE2_W	

	movlw	'K'			;ENTER "E".
	call	LCD_DISP_CHAR

	movlw	'I'			;ENTER "L".
	call	LCD_DISP_CHAR

	movlw	'R'			;ENTER "L".
	call	LCD_DISP_CHAR

	movlw	'K'			;ENTER "O".
	call	LCD_DISP_CHAR

	movlw	'S'			;ENTER "O".
	call	LCD_DISP_CHAR




	return	 
	
	
	

;===============================================================================
; DISPLAYLED  display key code on RA0,1,2,3
;===============================================================================
DISPLAY_LED
			movf	LASTKEY, W
			andlw	0x0F		; Make sure only 4 lower bits are valid
			movwf	PORTD
			call	DELAY20

			
			return


	 
;===============================================================================  
; DELAY ROUTINES START HERE.  
;===============================================================================  
DELAY255 
	movlw	0XFF		;DELAY 255 MS
	goto	D0
	
DELAY100 
	movlw	D'100'		;DELAY 100MS
	goto	D0
	
DELAY50	
	movlw	D'50'		;DELAY 50MS
	goto	D0
	
DELAY20	
	movlw	D'20'		;DELAY 20MS
	goto	D0
	
DELAY5	
	movlw	0X05		;DELAY 5.000 MS (4 MHZ CLOCK)
D0	movwf	COUNT1
D1	movlw	0XC7		;DELAY 1MS
	movwf	COUNTA
	movlw	0X01
	movwf	COUNTB
	
DELAY_0
	decfsz	COUNTA, F
	goto	$+2
	decfsz	COUNTB, F
	goto	DELAY_0

	decfsz	COUNT1	,F
	goto	D1
	retlw	0X00

	end
